@@ -54,26 +54,113 @@ static Lum::Def::AppInfo info;
 typedef Lum::Model::StdTable<Collection*>      CollectionsModel;
 typedef Lum::Base::Reference<CollectionsModel> CollectionsModelRef;
 
+class CollectionObject : public Lum::Object
+{
+private:
+  Lum::OS::FontRef font;
+  Collection       *collection;
+
+public:
+  CollectionObject()
+  : font(Lum::OS::display->GetFont())
+  {
+    // no code
+  }
+
+  void SetCollection(Collection* collection)
+  {
+    this->collection=collection;
+  }
+
+  void CalcSize()
+  {
+    minWidth=0;
+    minHeight=font->height;
+
+    width=minWidth;
+    height=minHeight;
+
+    Object::CalcSize();
+  }
+
+  void Draw(int x, int y, size_t w, size_t h)
+  {
+    Object::Draw(x,y,w,h);
+
+    if (!OIntersect(x,y,w,h)) {
+      return;
+    }
+
+    /* --- */
+
+    Lum::OS::DrawInfo *draw=GetDrawInfo();
+
+    draw->PushFont(font);
+    draw->PushForeground(parent->GetTextColor(draw));
+
+    //
+    // Name of collection
+    //
+
+    draw->DrawString(this->x,
+                     this->y+font->ascent,
+                     collection->GetTitle());
+
+    //
+    //
+    // Current level
+
+    std::wstring level;
+
+    if (collection->IsFinished()) {
+      level=L"finished";
+    }
+    else {
+      level=L"level "+
+            Lum::Base::NumberToWString(collection->GetCurrent()+1)+
+            L" of "+
+            Lum::Base::NumberToWString(collection->GetLevelCount());
+    }
+
+    draw->DrawString(this->x+width-font->StringWidth(level),
+                     this->y+font->ascent,
+                     level);
+
+    draw->PopFont();
+    draw->PopForeground();
+  }
+
+  static CollectionObject* Create(bool horizFlex, bool vertFlex)
+  {
+    CollectionObject *object=new CollectionObject();
+
+    object->SetFlex(horizFlex,vertFlex);
+
+    return object;
+  }
+};
+
 class CollectionDataProvider : public CollectionsModel::DataProvider
 {
+private:
+  CollectionObject *object;
+
 public:
-  std::wstring GetString(const CollectionsModel::Iterator& iter, size_t column) const
+  CollectionDataProvider()
+  : object(CollectionObject::Create(true,false))
   {
-    switch (column) {
-    case 1:
-      return (*iter)->GetTitle();
-    case 2:
-      if ((*iter)->IsFinished()) {
-        return L"finished";
-      }
-      else {
-        return Lum::Base::NumberToWString((*iter)->GetCurrent()+1)+
-               L"/"+
-               Lum::Base::NumberToWString((*iter)->GetLevelCount());
-      }
-    default:
-      assert(false);
-    }
+    // no code
+  }
+
+  ~CollectionDataProvider()
+  {
+    delete object;
+  }
+
+  Lum::Object* GetObject(const CollectionsModel::Iterator& iter, size_t column) const
+  {
+    object->SetCollection(*iter);
+    return object;
   }
 };
 
@@ -113,12 +200,11 @@ public:
     panel->SetHeight(Lum::Base::Size::pixel,(GetWindow()->GetParent()->GetHeight()*70)/100);
 
     headerModel=new Lum::Model::HeaderImpl();
-    headerModel->AddColumn(L"Collection",Lum::Base::Size::stdCharWidth,25,true);
-    headerModel->AddColumn(L"Current level",Lum::Base::Size::stdCharWidth,10);
+    headerModel->AddColumn(L"Collection",Lum::Base::Size::stdCharWidth,35,true);
 
     table=new Lum::Table();
     table->SetFlex(true,true);
-    table->SetShowHeader(true);
+    table->SetShowHeader(false);
     table->SetModel(list);
     table->SetHeaderModel(headerModel);
     table->SetSelection(selection);
@@ -250,19 +336,103 @@ public:
 typedef Lum::Model::StdRefTable<Theme,std::vector<Theme> > ThemesModel;
 typedef Lum::Base::Reference<ThemesModel>                  ThemesModelRef;
 
+class ThemeObject : public Lum::Object
+{
+private:
+  Lum::OS::FontRef font;
+  Theme            theme;
+
+public:
+  ThemeObject()
+  : font(Lum::OS::display->GetFont())
+  {
+    // no code
+  }
+
+  void SetTheme(const Theme& theme)
+  {
+    this->theme=theme;
+  }
+
+  void CalcSize()
+  {
+    minWidth=0;
+    minHeight=font->height;
+
+    width=minWidth;
+    height=minHeight;
+
+    Object::CalcSize();
+  }
+
+  void Draw(int x, int y, size_t w, size_t h)
+  {
+    Object::Draw(x,y,w,h);
+
+    if (!OIntersect(x,y,w,h)) {
+      return;
+    }
+
+    /* --- */
+
+    Lum::OS::DrawInfo *draw=GetDrawInfo();
+
+    draw->PushFont(font);
+    draw->PushForeground(parent->GetTextColor(draw));
+
+    //
+    // Name of theme
+    //
+
+    draw->DrawString(this->x,
+                     this->y+font->ascent,
+                     theme.name);
+
+    //
+    //
+    // Size of theme
+
+    std::wstring size=Lum::Base::NumberToWString(theme.boxSize)+L" pixel";
+
+    draw->DrawString(this->x+width-font->StringWidth(size),
+                     this->y+font->ascent,
+                     size);
+
+    draw->PopFont();
+    draw->PopForeground();
+  }
+
+  static ThemeObject* Create(bool horizFlex, bool vertFlex)
+  {
+    ThemeObject *object=new ThemeObject();
+
+    object->SetFlex(horizFlex,vertFlex);
+
+    return object;
+  }
+};
+
 class ThemeDataProvider : public ThemesModel::DataProvider
 {
+private:
+  ThemeObject *object;
+
 public:
-  std::wstring GetString(const ThemesModel::Iterator& iter, size_t column) const
+  ThemeDataProvider()
+  : object(ThemeObject::Create(true,false))
   {
-    switch (column) {
-    case 1:
-      return iter->name;
-    case 2:
-      return Lum::Base::NumberToWString(iter->boxSize);
-    default:
-      assert(false);
-    }
+    // no code
+  }
+
+  ~ThemeDataProvider()
+  {
+    delete object;
+  }
+
+  Lum::Object* GetObject(const ThemesModel::Iterator& iter, size_t column) const
+  {
+    object->SetTheme(*iter);
+    return object;
   }
 };
 
@@ -298,12 +468,11 @@ public:
     panel->SetHeight(Lum::Base::Size::pixel,(GetWindow()->GetParent()->GetHeight()*70)/100);
 
     headerModel=new Lum::Model::HeaderImpl();
-    headerModel->AddColumn(L"Theme name",Lum::Base::Size::stdCharWidth,25,true);
-    headerModel->AddColumn(L"box size",Lum::Base::Size::stdCharWidth,8);
+    headerModel->AddColumn(L"Theme",Lum::Base::Size::stdCharWidth,30,true);
 
     table=new Lum::Table();
     table->SetFlex(true,true);
-    table->SetShowHeader(true);
+    table->SetShowHeader(false);
     table->SetModel(list);
     table->SetHeaderModel(headerModel);
     table->SetSelection(selection);
